package tab

import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.Check
import androidx.compose.material.icons.outlined.Close
import androidx.compose.material.icons.outlined.Info
import androidx.compose.material.icons.outlined.Warning
import androidx.compose.material3.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import cafe.adriel.voyager.core.screen.Screen
import data.EGroup
import data.ENumber
import kotlinx.serialization.json.Json
import org.jetbrains.compose.resources.ExperimentalResourceApi
import veganenumbers.composeapp.generated.resources.Res.readBytes

class ENumberList : Screen {

    /**
     * Reads JSON file containing all e numbers
     * @return List of grouped E Numbers in `EGroup` type
     */
    @OptIn(ExperimentalResourceApi::class)
    private suspend fun getENumbers(): List<EGroup> {
        val bytes = readBytes("files/enumbers.json")

        println("Loading")

        val eNumbers = Json.decodeFromString<List<EGroup>>(bytes.decodeToString())

        println("Loaded")

        return eNumbers
    }

    @OptIn(ExperimentalFoundationApi::class)
    @Composable
    override fun Content() {
        val eNumbers = remember {
            mutableStateListOf<EGroup>()
        }
        println("Starting")

        LaunchedEffect(Unit) {
            eNumbers.addAll(getENumbers())
        }

        LazyColumn(contentPadding = PaddingValues(6.dp)) {
            eNumbers.forEach { eGroup ->
                stickyHeader {
                    Header(group = eGroup.group)
                }

                items(eGroup.numbers) { eNumber ->
                    ItemList(eNumber = eNumber)
                }
            }
        }
    }

    @Composable
    private fun Header(group: Int) {
        val text = if (group < 10) {
            "E${group}00 - E${group}99"
        } else {
            "E1000 - E1599"
        }

        Text(
            text = text,
            modifier = Modifier
                .fillMaxWidth()
                .background(MaterialTheme.colorScheme.surface)
                .padding(8.dp)
        )
    }

    @Composable
    private fun ItemList(eNumber: ENumber) {
        ListItem(
            headlineContent = {
                Text(text = "E${eNumber.number}")
            },
            leadingContent = {
                VeganStatusIcon(
                    vegan = eNumber.vegan,
                    notVegan = eNumber.notVegan
                )
            }
        )
        HorizontalDivider()
    }

    @Composable
    private fun VeganStatusIcon(vegan: Boolean, notVegan: Boolean) {
        val (icon, status) = if (vegan and !notVegan) {
            Pair(Icons.Outlined.Check, "Vegan")
        } else if (vegan and notVegan) {
            Pair(Icons.Outlined.Warning, "Potentially Not Vegan")
        } else if (!vegan and notVegan) {
            Pair(Icons.Outlined.Close, "Not Vegan")
        } else {
            Pair(Icons.Outlined.Info, "Unknown")
        }

        Icon(
            imageVector = icon,
            contentDescription = status
        )
    }
}
